/*
    Copyright (c) 2010, 2011 aizo ag, Zurich, Switzerland

    Author: Sergey 'Jin' Bostandzhyan <jin@dev.digitalstrom.org>

    This file is part of dS Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.
*/

#ifdef HAVE_CONFIG_H
    #include "autoconfig.h"
#endif


#ifdef HAVE_EXECINFO_H
    #include <execinfo.h>
    #include <stdlib.h>
#endif

#include <stdio.h>
#include "exceptions.h"
#include "log.h"

#define TRACE_TAG "_TRACE_"

namespace dSA
{

Exception::Exception(std::string message, int err, const char* file, int line,
                     const char* function) : m_message(message),
                                             m_error_code(err),
                                             m_file(file),
                                             m_line(line)
{
#if defined HAVE_BACKTRACE && defined HAVE_BACKTRACE_SYMBOLS
    void *b[100];
    int size = backtrace(b, 100);

    m_stackTrace.reserve(size);

    char **s = backtrace_symbols(b, size);
    for(int i = 0; i < size; i++)
    {
        std::string trace(s[i]);
        m_stackTrace.push_back(trace);
    }
    if (s)
    {
        free(s);
    }
#endif

}

Exception::Exception(std::string message) : m_message(message), m_line(-1)
{
#if defined HAVE_BACKTRACE && defined HAVE_BACKTRACE_SYMBOLS
    void *b[100];
    int size = backtrace(b, 100);

    m_stackTrace.reserve(size);

    char **s = backtrace_symbols(b, size);
    for(int i = 0; i < size; i++)
    {
        std::string trace(s[i]);
        m_stackTrace.push_back(trace);
    }

    if (s)
    {
        free(s);
    }
#endif

}

const std::string& Exception::getMessage() const
{
    return m_message;
}

int Exception::getErrorCode() const
{
    return m_error_code;
}


const std::vector<std::string> Exception::getStackTrace() const
{
    return m_stackTrace;
}

void Exception::printStackTrace()
{
    if (m_line >= 0)
    {
        FILE_LOG(logERROR) << "Exception raised in "
            << m_file << ":" << m_line << "] " << m_function << ": " <<
            m_message;
    }
    else
    {
        FILE_LOG(logERROR) <<  "Exception: " << m_message;
    }

#if defined HAVE_BACKTRACE && defined HAVE_BACKTRACE_SYMBOLS
    for (size_t i = 0; i < m_stackTrace.size(); i++)
    {
        std::string trace = m_stackTrace.at(i);
        FILE_LOG(logERROR) << TRACE_TAG << " " << i << ": " << trace;
    }
#endif // __CYGWIN__
}

SQLite3Exception::SQLite3Exception(std::string message, int err,
                                   const char* file, int line,
                                   const char* function) : Exception(
                                        message, err, file, line, function)
{}

SQLite3Exception::SQLite3Exception(std::string message) : Exception(message)
{}

} // namespace
